'use strict';

const {Branch} = require('../../models');
const {log, reportError} = require('../../core/utils');

let pages = {};
async function updatePagesRegistry() {
    const branches = await Branch.query();

    pages = {};
    branches.forEach(branch => {
        const wiki = branch.name;
        branch.pagesList.forEach(pageName => {
            if (pages[pageName]) {
                pages[pageName].push(wiki);
            } else {
                pages[pageName] = [wiki];
            }
        });
    });

    setTimeout(() => {
        initPagedRegistryUpdate(); // eslint-disable-line no-use-before-define
    }, 1000 * 60 * 30); // 30 min
}

function initPagedRegistryUpdate() {
    updatePagesRegistry().then(() => {
        log('InterWiki page registry updated');
    }).catch(error => {
        reportError(error, {message: 'Error occured during pages list registry update'});
    });
}

initPagedRegistryUpdate();

module.exports = function getWikiListByPageName(name) {
    return pages[name] || [];
};
