'use strict';

const {log, logToDiscord} = require('../core/utils');
const {
    getListCards,
    updateCardList,
    DAILY_BACKLOG_LIST_ID,
    DAILY_PREVIOUS_LIST_ID
} = require('../components/trello');
const {WikidotPage} = require('../models');
const {wk} = require('../components/wikidot-kit');

const PAGE_NAME_REG = /^[a-zA-Z0-9_:-]+$/;
const FEATURED_BLOCK_REGEXP = /\[!--\sFEATURED_DAILY_START\s--][\s\S]*\[!--\sFEATURED_DAILY_END\s--]/;
const WIKI_NAME = 'scp-ru';
const PAGE_NAME = '_default:main';

module.exports = async function run() {
    const previousCards = (await getListCards(DAILY_PREVIOUS_LIST_ID)).slice(0, 3);
    const backlogCards = await getListCards(DAILY_BACKLOG_LIST_ID);
    const current = backlogCards[0];

    if (!current) {
        throw new Error('Featured Daily Queue is empty');
    }

    const name = current.name;
    if (!PAGE_NAME_REG.test(name)) {
        throw new Error(`Featured Daily page name is not valid ("${name}")`);
    }

    log(`Rotating featured daily page to ${name}`);

    const currentPage = await WikidotPage.query().where({wikiName: WIKI_NAME, name}).first();
    const previousPages = await Promise.all(previousCards.map(async ({name: cardName}) => {
        return await WikidotPage.query().where({wikiName: WIKI_NAME, name: cardName}).first();
    }));

    const previousPagesLinks = previousPages.map(page => {
        return `- [/${page.name} ${page.title}]`;
    }).join('\n');

    const updatedBlock = `
[!-- FEATURED_DAILY_START --]
[[div class="scp-featured__block scp-featured__block_type_daily"]]
  [[div class="scp-featured__title"]]
    Объект дня
  [[/div]]
  [[div class="scp-featured__page-title"]]
    [/${currentPage.name} ${currentPage.title}]
  [[/div]]
  [[div class="scp-featured__content"]]
  [[/div]]

  [[div class="scp-featured__previous"]]
    [[span class="scp-featured__previous-title"]]Предыдущие дни:[[/span]]
      ${previousPagesLinks}
  [[/div]]
[[/div]]
[!-- FEATURED_DAILY_END --]
    `.trim();

    const content = (await wk.call(WIKI_NAME, 'pages.get_one', {page: PAGE_NAME})).content;
    const updatedContent = content.replace(FEATURED_BLOCK_REGEXP, updatedBlock);

    await wk.call(WIKI_NAME, 'pages.save_one', {
        page: PAGE_NAME,
        content: updatedContent
    });

    await updateCardList(current.id, DAILY_PREVIOUS_LIST_ID, (previousCards[0] && previousCards[0].pos - 1));

    logToDiscord(`Successfully rotated featured daily page to ${name}`);
};
