'use strict';

const {WikidotPage} = require('../models');
const {log, reportError} = require('../core/utils');
const cheerio = require('cheerio');
const Bluebird = require('bluebird');

const OBJECT_TITLE_REGEXP = /^SCP/;

module.exports = async function run() {
    log('Performing EN object titles extraction');

    try {
        const wikiName = 'scp-wiki';
        const pageNames = [
            'scp-series',
            'scp-series-2',
            'scp-series-3',
            'scp-series-4',
            'joke-scps',
            'archived-scps',
            'scp-ex'
        ];

        const listPages = await WikidotPage
            .query()
            .select('html')
            .where({wikiName})
            .where('name', 'in', pageNames);

        const html = listPages.reduce((result, page) => result + page.html, '');

        const $ = cheerio.load(html);
        const titledObjects = Array.from($('li')).reduce((objects, line) => {
            const text = $(line).text();
            if (OBJECT_TITLE_REGEXP.test(text)) {
                objects.push({
                    name: text.split(' - ')[0].toLowerCase(),
                    title: text
                });
            }
            return objects;
        }, []);

        await Bluebird.map(titledObjects, async (object) => {
            await WikidotPage
                .query()
                .patch({title: object.title})
                .where({wikiName, name: object.name});
        }, {concurrency: 10});

    } catch (error) {
        reportError(error, {message: 'Error extraction object titles'});
    }
};
