'use strict';

const {Branch} = require('../models');
const {log, reportError} = require('../core/utils');
const {wk} = require('../components/wikidot-kit');

module.exports = async function run() {
    log('Performing pages lists import');

    const branches = await Branch.query();

    await Promise.all(branches.map(async branch => {
        const name = branch.name;
        try {
            const pagesList = await wk.fetchPagesList(name);
            await Branch
                .query()
                .patch({pagesList})
                .where({name});
            log(`Pages list for wiki ${name} updated`);
        } catch (error) {
            reportError(error, {extra: {name}});
        }
    }));
    log('Pages list for all wikis updated');
};
